@@ -5,8 +5,7 @@ from ..responses import Response
 
 class DefaultHeadersMiddleware(Middleware):
     """Middleware to add the :attr:`app.config.default_headers
-    <baguette.Config.default_headers>` to every response.
-    """
+    <baguette.Config.default_headers>` to every response."""
 
     async def __call__(self, request: Request) -> Response:
         response = await self.next(request)
