@@ -1,3 +1,4 @@
+import copy
 import json
 import typing
 from cgi import parse_header
@@ -6,8 +7,9 @@ from urllib.parse import parse_qs
 from .forms import Field, Form, MultipartForm, URLEncodedForm
 from .headers import Headers
 from .httpexceptions import BadRequest
-from .types import ASGIApp, JSONType, Receive, Scope
-from .utils import get_encoding_from_headers
+from .json import UJSONDecoder, UJSONEncoder
+from .types import ASGIApp, JSONType, Receive, Scope, StrOrBytes
+from .utils import get_encoding_from_headers, to_bytes, to_str
 
 FORM_CONTENT_TYPE = ["application/x-www-form-urlencoded", "multipart/form-data"]
 
@@ -76,7 +78,6 @@ class Request:
 
         encoding: :class:`str`
             Encoding of the response body.
-
     """
 
     def __init__(self, app: ASGIApp, scope: Scope, receive: Receive):
@@ -105,6 +106,15 @@ class Request:
         )[0]
         self.encoding: str = get_encoding_from_headers(self.headers) or "utf-8"
 
+        # cached
+        self._raw_body: bytes = None
+        self._body: str = None
+        self._json: JSONType = None
+        self._form: Form = None
+
+    # --------------------------------------------------------------------------
+    # Body methods
+
     async def raw_body(self) -> bytes:
         """Gets the raw request body in :class:`bytes`.
 
@@ -112,11 +122,10 @@ class Request:
         -------
             :class:`bytes`
                 Raw request body.
-
         """
 
         # caching
-        if hasattr(self, "_raw_body"):
+        if self._raw_body is not None:
             return self._raw_body
 
         body = b""
@@ -137,11 +146,10 @@ class Request:
         -------
             :class:`str`
                 Request body.
-
         """
 
         # caching
-        if hasattr(self, "_body"):
+        if self._body is not None:
             return self._body
 
         raw_body = await self.raw_body()
@@ -162,17 +170,16 @@ class Request:
                 If the JSON body is not JSON.
                 You can usually not handle this error as it will be handled by
                 the app and converted to a response with a ``400`` status code.
-
         """
 
         # caching
-        if hasattr(self, "_json"):
+        if self._json is not None:
             return self._json
 
         body = await self.body()
         try:
-            self._json = json.loads(body)
-        except json.JSONDecodeError:
+            self._json = json.loads(body, cls=UJSONDecoder)
+        except (json.JSONDecodeError, ValueError):
             raise BadRequest(description="Can't decode body as JSON")
         return self._json
 
@@ -188,26 +195,30 @@ class Request:
         -------
             :class:`~baguette.forms.Form`
                 Parsed form.
-
         """
 
-        body = await self.raw_body()
-        if self.content_type not in FORM_CONTENT_TYPE:
-            raise ValueError(
-                "Content-type '{}' isn't one of: {}".format(
-                    self.content_type, ", ".join(FORM_CONTENT_TYPE)
+        if self._form is None:
+            body = await self.raw_body()
+            if self.content_type not in FORM_CONTENT_TYPE:
+                raise ValueError(
+                    "Content-type '{}' isn't one of: {}".format(
+                        self.content_type, ", ".join(FORM_CONTENT_TYPE)
+                    )
                 )
-            )
 
-        if self.content_type == "application/x-www-form-urlencoded":
-            form = URLEncodedForm.parse(body, encoding=self.encoding)
-        elif self.content_type == "multipart/form-data":
-            params = parse_header(self.headers.get("content-type", ""))[1]
-            form = MultipartForm.parse(
-                body,
-                boundary=params["boundary"].encode(self.encoding),
-                encoding=self.encoding,
-            )
+            if self.content_type == "application/x-www-form-urlencoded":
+                form = URLEncodedForm.parse(body, encoding=self.encoding)
+            elif self.content_type == "multipart/form-data":
+                params = parse_header(self.headers.get("content-type", ""))[1]
+                form = MultipartForm.parse(
+                    body,
+                    boundary=params["boundary"].encode(self.encoding),
+                    encoding=self.encoding,
+                )
+
+            self._form = form
+        else:
+            form = self._form.copy()
 
         if include_querystring:
             for name, values in self.querystring.items():
@@ -217,3 +228,82 @@ class Request:
                     form.fields[name] = Field(name, values)
 
         return form
+
+    # --------------------------------------------------------------------------
+    # Setters
+
+    def set_raw_body(self, raw_body: bytes):
+        """Sets the raw body of the request.
+
+        Arguments
+        ---------
+            raw_body : :class:`bytes`
+                The new request raw body
+
+        Raises
+        ------
+            TypeError
+                The raw body isn't of type :class:`bytes`
+        """
+
+        if not isinstance(raw_body, bytes):
+            raise TypeError(
+                "Argument raw_body most be of type bytes. Got "
+                + raw_body.__class__.__name__
+            )
+        self._raw_body = raw_body
+
+    def set_body(self, body: StrOrBytes):
+        """Sets the request body.
+
+        Parameters
+        ----------
+            body : :class:`str` or :class:`bytes`
+                The new request body
+
+        Raises
+        ------
+            TypeError
+                The body isn't of type :class:`str` or :class:`bytes`
+        """
+
+        self._body = to_str(body)
+        self.set_raw_body(to_bytes(body))
+
+    def set_json(self, data: JSONType):
+        """Sets the request JSON data.
+
+        Parameters
+        ----------
+            data : Anything JSON serializable
+                The data to put in the request body.
+
+        Raises
+        ------
+            TypeError
+                The data isn't JSON serializable.
+        """
+
+        self.set_body(json.dumps(data, cls=UJSONEncoder))
+        self._json = copy.deepcopy(data)
+
+    def set_form(self, form: Form):
+        """Sets the request form.
+
+        Parameters
+        ----------
+            form : :class:`~baguette.forms.Form`
+                The form to add to the request.
+
+        Raises
+        ------
+            TypeError
+                The form isn't a :class:`~baguette.forms.Form`.
+        """
+
+        if not isinstance(form, Form):
+            raise TypeError(
+                "Argument form most be of type Form. Got "
+                + form.__class__.__name__
+            )
+        self._form = form.copy()
