@@ -5,20 +5,16 @@ import typing
 import zlib
 
 import aiofiles
-import ujson
 
 from .headers import Headers, make_headers
 from .httpexceptions import HTTPException, NotFound
-from .types import HeadersType, Result, Send
-from .utils import safe_join
+from .json import UJSONEncoder
+from .types import HeadersType, Result, Send, StrOrBytes
+from .utils import safe_join, to_bytes, to_str
 
 HTML_TAG_REGEX = re.compile(r"<\s*\w+[^>]*>.*?<\s*/\s*\w+\s*>")
 
 
-class UJSONEncoder(json.JSONEncoder):
-    encode = ujson.encode
-
-
 class Response:
     """Base response class.
 
@@ -36,12 +32,6 @@ class Response:
 
     Attributes
     ----------
-        body : :class:`bytes`
-            The raw response body.
-
-        text : :class:`str`
-            The response body.
-
         status_code : :class:`int`
             The HTTP status code of the reponse.
 
@@ -53,24 +43,49 @@ class Response:
 
     def __init__(
         self,
-        body: typing.Union[str, bytes],
+        body: StrOrBytes,
         status_code: int = 200,
         headers: typing.Optional[HeadersType] = None,
     ):
-        if isinstance(body, str):
-            self.text: str = body
-            self.body: bytes = body.encode(self.CHARSET)
-        elif isinstance(body, bytes):
-            self.text: str = body.decode(self.CHARSET)
-            self.body: bytes = body
-        else:
-            raise ValueError("body must be str or bytes")
-
+        self.body = body
         self.status_code = status_code
         self.headers = make_headers(headers)
 
+    @property
+    def body(self) -> str:
+        """The reponse body.
+
+        .. note::
+            Setting the request body also accepts a :class:`bytes` but accessing
+            the request body will always return a :class:`str`.
+        """
+
+        return self._body
+
+    @body.setter
+    def body(self, body: StrOrBytes):
+        self._raw_body: bytes = to_bytes(body, encoding=self.CHARSET)
+        self._body: str = to_str(body, encoding=self.CHARSET)
+
+    @property
+    def raw_body(self) -> bytes:
+        """The reponse raw body.
+
+        .. note::
+            Setting the request body also accepts a :class:`str` but accessing
+            the request raw body will always return a :class:`bytes`.
+        """
+
+        return self._raw_body
+
+    @raw_body.setter
+    def raw_body(self, body: StrOrBytes):
+        self._raw_body: bytes = to_bytes(body, encoding=self.CHARSET)
+        self._body: str = to_str(body, encoding=self.CHARSET)
+
     async def _send(self, send: Send):
         """Sends the response."""
+
         await send(
             {
                 "type": "http.response.start",
@@ -81,7 +96,7 @@ class Response:
         await send(
             {
                 "type": "http.response.body",
-                "body": self.body,
+                "body": self.raw_body,
             }
         )
 
@@ -103,26 +118,17 @@ class JSONResponse(Response):
 
     Attributes
     ----------
-        body : :class:`bytes`
-            The raw response body.
-
-        text : :class:`str`
-            The response body.
-
-        json : Anything JSON serializable
-            The response JSON body.
+        JSON_ENCODER : JSON encoder
+            The JSON encoder to use in :func:`json.dumps` with the ``cls``
+            keyword argument. This is a class attribute.
+            Default: the encoder from
+            `ujson <https://github.com/ultrajson/ultrajson>`_
 
         status_code : :class:`int`
             The HTTP status code of the reponse.
 
         headers : :class:`Headers`
             The headers of the reponse.
-
-        JSON_ENCODER : JSON encoder
-            The JSON encoder to use in :func:`json.dumps` with the ``cls``
-            keyword argument. This is a class attribute.
-            Default: the encoder from
-            `ujson <https://github.com/ultrajson/ultrajson>`_
     """
 
     JSON_ENCODER = UJSONEncoder
@@ -134,15 +140,48 @@ class JSONResponse(Response):
         headers: typing.Optional[HeadersType] = None,
     ):
         self.json = data
-        body: str = json.dumps(data, cls=self.JSON_ENCODER)
-        super().__init__(body, status_code, headers)
+        super().__init__(self.body, status_code, headers)
         self.headers["content-type"] = "application/json"
 
+    @property
+    def json(self) -> typing.Any:
+        """The request JSON data."""
+
+        return self._json
+
+    @json.setter
+    def json(self, data: typing.Any):
+        self.body = json.dumps(data, cls=self.JSON_ENCODER)
+        self._json = data
+
 
 class PlainTextResponse(Response):
+    """Plain text response class.
+
+    Arguments
+    ---------
+        body : :class:`str` or :class:`bytes`
+            The response body.
+
+        status_code : :class:`int`
+            The HTTP status code of the reponse.
+
+        headers : :class:`list` of ``(str, str)`` tuples, \
+        :class:`dict` or :class:`Headers`
+            The headers of the reponse.
+
+    Attributes
+    ----------
+        status_code : :class:`int`
+            The HTTP status code of the reponse.
+
+        headers : :class:`Headers`
+            The headers of the reponse.
+    """
+
     def __init__(
         self,
-        text: typing.Union[str, bytes],
+        text: StrOrBytes,
         status_code: int = 200,
         headers: typing.Optional[HeadersType] = None,
     ):
@@ -151,9 +190,32 @@ class PlainTextResponse(Response):
 
 
 class HTMLResponse(Response):
+    """HTML response class.
+
+    Arguments
+    ---------
+        body : :class:`str` or :class:`bytes`
+            The response body.
+
+        status_code : :class:`int`
+            The HTTP status code of the reponse.
+
+        headers : :class:`list` of ``(str, str)`` tuples, \
+        :class:`dict` or :class:`Headers`
+            The headers of the reponse.
+
+    Attributes
+    ----------
+        status_code : :class:`int`
+            The HTTP status code of the reponse.
+
+        headers : :class:`Headers`
+            The headers of the reponse.
+    """
+
     def __init__(
         self,
-        html: typing.Union[str, bytes],
+        html: StrOrBytes,
         status_code: int = 200,
         headers: typing.Optional[HeadersType] = None,
     ):
@@ -162,6 +224,29 @@ class HTMLResponse(Response):
 
 
 class EmptyResponse(PlainTextResponse):
+    """Empty response class.
+
+    Arguments
+    ---------
+        body : :class:`str` or :class:`bytes`
+            The response body.
+
+        status_code : :class:`int`
+            The HTTP status code of the reponse.
+
+        headers : :class:`list` of ``(str, str)`` tuples, \
+        :class:`dict` or :class:`Headers`
+            The headers of the reponse.
+
+    Attributes
+    ----------
+        status_code : :class:`int`
+            The HTTP status code of the reponse.
+
+        headers : :class:`Headers`
+            The headers of the reponse.
+    """
+
     def __init__(
         self,
         status_code: int = 204,
@@ -171,13 +256,53 @@ class EmptyResponse(PlainTextResponse):
 
 
 class RedirectResponse(Response):
+    """Redirect response class.
+
+    Arguments
+    ---------
+        location: :class:`str` or :class:`bytes`
+            The location to redirect the request to.
+
+        body : :class:`str` or :class:`bytes`
+            The response body.
+
+        status_code: :class:`int`
+            Status code of the redirect response.
+            Default: ``301``.
+
+        headers: :class:`list` of ``(str, str)`` tuples, \
+        :class:`dict` or :class:`Headers`
+            Headers to include in the response.
+            Any location header will be overwritten with the location parameter.
+            Default: ``None``.
+
+    Attributes
+    ----------
+        status_code : :class:`int`
+            The HTTP status code of the reponse.
+
+        headers : :class:`Headers`
+            The headers of the reponse.
+    """
+
     def __init__(
         self,
         location: str,
+        body: StrOrBytes = "",
         status_code: int = 301,
         headers: typing.Optional[HeadersType] = None,
     ):
-        super().__init__("", status_code, headers)
+        super().__init__(body=body, status_code=status_code, headers=headers)
+        self.location = location
+
+    @property
+    def location(self):
+        """The location to redirect the request to."""
+
+        return self.headers["location"]
+
+    @location.setter
+    def location(self, location):
         self.headers["location"] = location
 
 
@@ -287,12 +412,7 @@ def make_response(result: Result) -> Response:
     if isinstance(body, (list, dict)):
         response = JSONResponse(body, status_code or 200, headers)
     elif isinstance(body, (str, bytes)):
-        if (
-            HTML_TAG_REGEX.search(
-                body.decode() if isinstance(body, bytes) else body
-            )
-            is not None
-        ):
+        if HTML_TAG_REGEX.search(to_str(body, encoding="ascii")) is not None:
             response = HTMLResponse(body, status_code or 200, headers)
         else:
             response = PlainTextResponse(body, status_code or 200, headers)
@@ -386,14 +506,15 @@ def redirect(
 
     Arguments
     ---------
-        location: :class:`str`
+        location: :class:`str` or :class:`bytes`
             The location to redirect the request to.
 
         status_code: :class:`int`
             Status code of the redirect response.
             Default: ``301``.
 
-        headers: anything accepted by :func:`make_headers`
+        headers: :class:`list` of ``(str, str)`` tuples, \
+        :class:`dict` or :class:`Headers`
             Headers to include in the response.
             Any location header will be overwritten with the location parameter.
             Default: ``None``.
@@ -403,4 +524,7 @@ def redirect(
         :class:`RedirectResponse`
             The created redirect response.
     """
-    return RedirectResponse(location, status_code, headers)
+
+    return RedirectResponse(
+        location=location, status_code=status_code, headers=headers
+    )
