@@ -14,10 +14,13 @@ Send = typing.Callable[[Message], typing.Awaitable[None]]
 
 ASGIApp = typing.Callable[[Scope, Receive, Send], typing.Awaitable[None]]
 
+StrOrBytes = typing.Union[str, bytes]
+
 HeadersType = typing.Union[
     "Headers",
-    typing.Mapping[str, str],
-    typing.Sequence[typing.Tuple[str, str]],
+    StrOrBytes,
+    typing.Mapping[StrOrBytes, StrOrBytes],
+    typing.Sequence[typing.Tuple[StrOrBytes, StrOrBytes]],
 ]
 
 Result = typing.Union[
